@@ -50,6 +50,14 @@ class edXinclude(Base.Command):		# include external XML file
 class edXincludepy(Base.Command):		# include external python file (puts inside <script>)
     args = 'self'
 
+class edXdndtex(Base.Command):		# insert external drag-and-drop problem (should point to latex2dnd tex file)
+    args = '[ attrib_string ] self'
+    def invoke(self, tex):
+        Command.invoke(self, tex)
+        self.attributes['filename'] = tex.filename
+        self.attributes['linenum'] = tex.lineNumber
+        print "  --> edXdndtex in %s: dndtex=%s, line=%s" % (tex.filename, self.attributes['self'], tex.lineNumber)
+
 class edXshowhide(Base.Environment):	# block of text to be hidden by default, but with clickable "show"
     args = ' { id } { description } self'
 
@@ -71,8 +79,11 @@ class edXjavascript(Base.verbatim):
 class endedXjavascript(Base.endverbatim):
     macroName = "endedXjavascript"
 
-class edXmath(Base.Environment):
-    args = 'self'
+class edXmath(Base.verbatim):
+    macroName = "edXmath"
+
+class endedXmath(Base.endverbatim):
+    macroName = "endedXmath"
 
 class edXxml(Base.Command):
     args = 'self'
