@@ -112,7 +112,7 @@ class XBundle(object):
 
     DescriptorTags = ['course','chapter','sequential','vertical','html','problem','video',
                       'conditional', 'combinedopenended', 'videosequence', 'problemset',
-                      'wrapper', 'poll_question', 'randomize', 'proctor' ]
+                      'wrapper', 'poll_question', 'randomize', 'proctor', 'discussion' ]
     KeepTogetherTags = []	# for latex2edx - simplier course structure
     MapTags = dict(section='sequential')
     DefaultSemester = '2013_Fall'
